/// <reference types="sequelize" />
import * as Sequelize from 'sequelize';
import { Spec } from 'pretur.spec';
import { Resolver, UnitializedResolver } from './resolver';
import { Synchronizer, UnitializedSynchronizer } from './synchronizer';
import { UninitializedSequelizeModel, SequelizeModel } from './sequelizeModel';
import { Pool } from './pool';
export declare type FieldWhereClause<T> = T | Sequelize.WhereOptions | Sequelize.WhereOptions | Sequelize.col | Sequelize.and | Sequelize.or | Sequelize.WhereGeometryOptions | T[] | object;
export declare type AliasModelMap<T> = {
    [P in keyof T]: string;
};
export declare type AliasKeyMap<T> = {
    [P in keyof T]: keyof T;
};
export declare type FieldWhereBuilders<T> = {
    [P in keyof T]?: (value: T[P]) => FieldWhereClause<T[P]>;
};
export interface ModelDescriptor<T extends object> {
    spec: Spec<T>;
    name: string;
    primaryKeys: (keyof T)[];
    sequelizeModel?: SequelizeModel<T>;
    resolver?: Resolver<T>;
    synchronizer?: Synchronizer<T>;
    aliasModelMap: AliasModelMap<T>;
    aliasKeyMap: AliasKeyMap<T>;
    allowedAttributes: (keyof T)[];
    mutableAttributes: (keyof T)[];
    defaultOrder: [keyof T, 'ASC' | 'DESC'];
    fieldWhereBuilders?: FieldWhereBuilders<T>;
    sanitizeAttributes(attributes?: (keyof T)[] | keyof T): (keyof T)[];
    initialize(pool: Pool): void;
}
export interface BuildModelDescriptorOptions<T> {
    sequelizeModel?: UninitializedSequelizeModel<T>;
    resolver?: UnitializedResolver<T>;
    synchronizer?: UnitializedSynchronizer<T>;
    defaultOrder?: [keyof T, 'ASC' | 'DESC'];
    fieldWhereBuilders?: FieldWhereBuilders<T>;
    allowedAttributes?: (keyof T)[];
    allowedMutableAttributes?: (keyof T)[];
}
export declare function buildModelDescriptor<T extends object>(spec: Spec<T>, options?: BuildModelDescriptorOptions<T>): ModelDescriptor<T>;
