/// <reference types="bluebird" />
/// <reference types="sequelize" />
import * as Bluebird from 'bluebird';
import * as Sequelize from 'sequelize';
import { I18nBundle } from 'pretur.i18n';
import { Spec } from 'pretur.spec';
import { Pool } from './pool';
import { InsertMutateRequest, UpdateMutateRequest, RemoveMutateRequest, MutateRequest } from 'pretur.sync';
export interface ResultItemAppender {
    appendError(error: I18nBundle): void;
    appendWarning(warning: I18nBundle): void;
}
export interface ErrorHandler<T> {
    (data: T, error: any, rip: ResultItemAppender): Bluebird<void>;
}
export interface Insert<T> {
    (transaction: Sequelize.Transaction, item: InsertMutateRequest<T>, rip: ResultItemAppender, context: any): Bluebird<Partial<T> | void>;
}
export interface Update<T> {
    (transaction: Sequelize.Transaction, item: UpdateMutateRequest<T>, rip: ResultItemAppender, context: any): Bluebird<void>;
}
export interface Remove<T> {
    (transaction: Sequelize.Transaction, item: RemoveMutateRequest<T>, rip: ResultItemAppender, context: any): Bluebird<void>;
}
export interface Synchronizer<T> {
    (transaction: Sequelize.Transaction, item: MutateRequest<T>, rip: ResultItemAppender, context: any): Bluebird<Partial<T> | void>;
}
export interface UnitializedSynchronizer<T> {
    synchronizer: Synchronizer<T>;
    initialize(pool: Pool): void;
}
export interface SynchronizationInterceptor<T, R> {
    (transaction: Sequelize.Transaction, item: T, rip: ResultItemAppender, pool: Pool, context: any): Bluebird<R>;
}
export interface BuildSynchronizerOptions<T> {
    insertErrorHandler?: ErrorHandler<InsertMutateRequest<T>>;
    updateErrorHandler?: ErrorHandler<UpdateMutateRequest<T>>;
    removeErrorHandler?: ErrorHandler<RemoveMutateRequest<T>>;
    insertInterceptor?: SynchronizationInterceptor<InsertMutateRequest<T>, Partial<T> | boolean>;
    updateInterceptor?: SynchronizationInterceptor<UpdateMutateRequest<T>, boolean>;
    removeInterceptor?: SynchronizationInterceptor<RemoveMutateRequest<T>, boolean>;
}
export declare function buildSynchronizer<T extends object>(spec: Spec<T>, options?: BuildSynchronizerOptions<T>): UnitializedSynchronizer<T>;
